import unittest

from sacrerouge.data import MetricsDict


class TestMetricsDict(unittest.TestCase):
    def test_get_set_item(self):
        metrics = MetricsDict()

        metrics['a'] = 4
        assert metrics['a'] == 4

        metrics['b']['c'] = [1, 2]
        assert metrics['b']['c'] == [1, 2]
        assert isinstance(metrics['b'], MetricsDict)

        metrics['d'] = {'e': 4, 'f': {'g': 4}}
        assert metrics['d'] == {'e': 4, 'f': {'g': 4}}
        assert isinstance(metrics['d'], MetricsDict)
        assert isinstance(metrics['d']['f'], MetricsDict)

    def test_add(self):
        a = MetricsDict({'k1': 1, 'k2': {'k3': 4}})
        b = MetricsDict({'k1': 2, 'k2': {'k3': 5}})
        c = MetricsDict({'k1': 3, 'k2': {'k3': 6}})

        assert a + b == {'k1': 3, 'k2': {'k3': 9}}
        assert a == {'k1': 1, 'k2': {'k3': 4}}
        assert b == {'k1': 2, 'k2': {'k3': 5}}

        assert sum([a, b, c]) == {'k1': 6, 'k2': {'k3': 15}}
        assert a == {'k1': 1, 'k2': {'k3': 4}}
        assert b == {'k1': 2, 'k2': {'k3': 5}}
        assert c == {'k1': 3, 'k2': {'k3': 6}}

    def test_divide(self):
        metrics = MetricsDict({'k1': 1, 'k2': {'k3': 4}})
        assert metrics / 2 == {'k1': 0.5, 'k2': {'k3': 2.0}}
        assert metrics == {'k1': 1, 'k2': {'k3': 4}}

    def test_average_values(self):
        metrics = MetricsDict({'k1': 1, 'k2': {'k3': [1, 2, 3]}})
        assert metrics.average_values() == {'k1': 1, 'k2': {'k3': 2.0}}
        assert metrics == {'k1': 1, 'k2': {'k3': [1, 2, 3]}}

    def test_flatten_keys(self):
        metrics = MetricsDict({'k1': 1, 'k2': {'k3': 4}})
        assert metrics.flatten_keys() == {'k1': 1, 'k2_k3': 4}
        assert metrics == {'k1': 1, 'k2': {'k3': 4}}

    def test_init_with_metrics_dict(self):
        a = MetricsDict({'k1': 1, 'k2': {'k3': [1, 2, 3]}})
        b = MetricsDict(a)

        b['k2']['k3'].append(4)
        assert a == {'k1': 1, 'k2': {'k3': [1, 2, 3]}}
        assert b == {'k1': 1, 'k2': {'k3': [1, 2, 3, 4]}}

    def test_update(self):
        m1 = MetricsDict({'k1': 1, 'k2': {'k3': [1, 2, 3]}})
        m2 = MetricsDict({'k4': 4, 'k2': {'k3': 5, 'k5': 8}})
        m1.update(m2)
        assert m1 == {'k1': 1, 'k2': {'k3': 5, 'k5': 8}, 'k4': 4}

    def test_approx_equal(self):
        m1 = MetricsDict({'k1': 1, 'k2': {'k3': [1, 2, 3]}})
        m2 = MetricsDict({'k1': 1.01, 'k2': {'k3': [1.01, 2.01, 3.01]}})
        assert m1.approx_equal(m2, abs=0.02)

        # Wrong keys
        m1 = MetricsDict({'k1': 1})
        m2 = MetricsDict({'k2': 1})
        assert not m1.approx_equal(m2, abs=0.02)

        # Wrong types
        m1 = MetricsDict({'k1': 1, 'k2': {'k3': 1.0}})
        m2 = MetricsDict({'k1': 1.01, 'k2': {'k3': [1.01, 2.01, 3.01]}})
        assert not m1.approx_equal(m2, abs=0.02)

        # Too different
        m1 = MetricsDict({'k1': 1})
        m2 = MetricsDict({'k1': 1.5})
        assert not m1.approx_equal(m2, abs=0.02)




import tarfile
from collections import defaultdict
from nltk.tokenize import sent_tokenize

from sacrerouge.io import JsonlWriter


def load_reference_summaries(eval_path: str):
    summaries = defaultdict(dict)
    with tarfile.open(eval_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('Evaluation/2017_summaries/') and member.name.endswith('.txt'):
                if 'simple' in member.name:
                    # Seems to be a dummy language
                    continue
                path = member.name.split('/')
                language_code = path[2]
                instance_id = path[3][:32]
                text = tar.extractfile(member).read().decode().splitlines()
                if language_code == 'en':
                    text = [sent_tokenize(paragraph) for paragraph in text]
                summaries[language_code][instance_id] = {
                    'summarizer_id': 'reference',
                    'summarizer_type': 'reference',
                    'text': text
                }
    return summaries


def load_peer_summaries(eval_path: str):
    summaries = defaultdict(lambda: defaultdict(dict))
    with tarfile.open(eval_path, 'r') as tar:
        for member in tar.getmembers():
            if member.name.startswith('Evaluation/MultiLing17_Submissions') and member.name.endswith('.txt'):
                path = member.name.split('/')
                if path[2] == 'baselines2017':
                    if member.name.endswith('_body.txt'):
                        # Skip the "body" baseline"
                        continue
                    summarizer_id = path[3]
                    if summarizer_id == '2017_summaries':
                        # These are the references again
                        continue
                    language = path[4]
                    if language == 'simple':
                        continue

                    instance_id = path[5][:32]
                else:
                    summarizer_id = path[2] + '-' + path[3]
                    language = path[4]
                    instance_id = path[5][:32]

                text = tar.extractfile(member).read().decode().splitlines()
                if language == 'en':
                    text = [sent_tokenize(paragraph) for paragraph in text]
                summaries[language][instance_id][summarizer_id] = {
                    'summarizer_id': summarizer_id,
                    'summarizer_type': 'peer',
                    'text': text
                }
    return summaries


def save_data(references, peers, output_dir):
    for language in references.keys():
        with JsonlWriter(f'{output_dir}/{language}.summaries.jsonl') as out:
            for instance_id in sorted(references[language].keys()):
                for summarizer_id in sorted(peers[language][instance_id].keys()):
                    summary = peers[language][instance_id][summarizer_id]
                    out.write({
                        'instance_id': instance_id,
                        'summarizer_id': summarizer_id,
                        'summarizer_type': summary['summarizer_type'],
                        'summary': summary,
                        'references': [references[language][instance_id]]
                    })


def setup(eval_path: str, output_dir: str):
    references = load_reference_summaries(eval_path)
    peers = load_peer_summaries(eval_path)
    save_data(references, peers, output_dir)


import os
import pytest
import unittest

from sacrerouge.commands.correlate import aggregate_metrics
from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_metrics_file_path = 'datasets/duc-tac/duc2005/v1.0/task1.metrics.jsonl'


class TestDUC2005SystemLevel(unittest.TestCase):
    @pytest.mark.skipif(not os.path.exists(_metrics_file_path), reason='DUC 2005 metrics file does not exist')
    def test_system_level(self):
        summary_level_metrics = JsonlReader(_metrics_file_path, Metrics).read()
        system_level_metrics = aggregate_metrics(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # ROUGE/rouge2.jk.m.avg
        assert system_level_metrics['32']['rouge-1_jk']['recall'] == pytest.approx(32.4835488888889, 1e-2)
        assert system_level_metrics['32']['rouge-1_jk']['precision'] == pytest.approx(33.3402216666667, 1e-2)
        assert system_level_metrics['21']['rouge-2_jk']['recall'] == pytest.approx(5.72633166666666, 1e-2)
        assert system_level_metrics['21']['rouge-2_jk']['precision'] == pytest.approx(5.70379111111111, 1e-2)
        assert system_level_metrics['E']['rouge-2_jk']['recall'] == pytest.approx(10.5482258064516, 1e-2)
        assert system_level_metrics['E']['rouge-2_jk']['precision'] == pytest.approx(11.9319677419355, 1e-2)

        # ROUGE/rougeSU4.jk.m.avg
        assert system_level_metrics['H']['rouge-su4_jk']['recall'] == pytest.approx(14.8429, 1e-2)
        assert system_level_metrics['F']['rouge-su4_jk']['recall'] == pytest.approx(15.8717666666667, 1e-2)
        assert system_level_metrics['E']['rouge-su4_jk']['recall'] == pytest.approx(15.9369677419355, 1e-2)
        assert system_level_metrics['23']['rouge-su4_jk']['recall'] == pytest.approx(5.56854166666667, 1e-2)
        assert system_level_metrics['1']['rouge-su4_jk']['recall'] == pytest.approx(8.71616333333333, 1e-2)

        # Not checking responsiveness because they implemented a scaled responsiveness which
        # we did not do here



import os
import pytest
import unittest
from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_task1_metrics_file_path = 'datasets/duc-tac/duc2007/v1.0/task1.metrics.jsonl'
_task2_metrics_file_path = 'datasets/duc-tac/duc2007/v1.0/task2.A-B-C.metrics.jsonl'


class TestDUC2007Metrics(unittest.TestCase):
    def _convert_to_dicts(self, metrics_list: List[Metrics]) -> Dict[str, Dict[str, Metrics]]:
        metrics_dicts = defaultdict(dict)
        for metrics in metrics_list:
            metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics.metrics
        return metrics_dicts

    @pytest.mark.skipif(not os.path.exists(_task1_metrics_file_path), reason='DUC 2007 task 1 metrics file does not exist')
    def test_task1_metrics(self):
        summary_level_metrics = JsonlReader(_task1_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files. Unlike
        # the "system_level_test.py", these metrics don't have aggregated values in the NIST data,
        # so we just check to make sure a few of them were correctly parsed
        # mainPyramidEval/scoring/2007_modified_scores.txt
        assert summary_level_metrics['d0714']['23']['modified_pyramid_score'] == pytest.approx(0.3636, 1e-2)
        assert summary_level_metrics['d0710']['29']['modified_pyramid_score'] == pytest.approx(0.5062, 1e-2)
        assert summary_level_metrics['d0728']['2']['modified_pyramid_score'] == pytest.approx(0.04, 1e-2)

        assert summary_level_metrics['d0714']['23']['num_scus'] == 10
        assert summary_level_metrics['d0710']['29']['num_scus'] == 18
        assert summary_level_metrics['d0728']['2']['num_scus'] == 1

        assert summary_level_metrics['d0714']['23']['num_repetitions'] == 0
        assert summary_level_metrics['d0710']['29']['num_repetitions'] == 2
        assert summary_level_metrics['d0728']['2']['num_repetitions'] == 0

        assert summary_level_metrics['d0714']['23']['linguistic_quality']['Q1'] == 5
        assert summary_level_metrics['d0710']['29']['linguistic_quality']['Q1'] == 3
        assert summary_level_metrics['d0728']['2']['linguistic_quality']['Q1'] == 2

        assert summary_level_metrics['d0714']['23']['linguistic_quality']['Q2'] == 5
        assert summary_level_metrics['d0710']['29']['linguistic_quality']['Q2'] == 2
        assert summary_level_metrics['d0728']['2']['linguistic_quality']['Q2'] == 3

        assert summary_level_metrics['d0714']['23']['linguistic_quality']['Q3'] == 4
        assert summary_level_metrics['d0710']['29']['linguistic_quality']['Q3'] == 2
        assert summary_level_metrics['d0728']['2']['linguistic_quality']['Q3'] == 3

        assert summary_level_metrics['d0714']['23']['linguistic_quality']['Q4'] == 5
        assert summary_level_metrics['d0710']['29']['linguistic_quality']['Q4'] == 3
        assert summary_level_metrics['d0728']['2']['linguistic_quality']['Q4'] == 3

        assert summary_level_metrics['d0714']['23']['linguistic_quality']['Q5'] == 4
        assert summary_level_metrics['d0710']['29']['linguistic_quality']['Q5'] == 2
        assert summary_level_metrics['d0728']['2']['linguistic_quality']['Q5'] == 3

        assert summary_level_metrics['d0714']['23']['content_responsiveness'] == 3
        assert summary_level_metrics['d0710']['29']['content_responsiveness'] == 3
        assert summary_level_metrics['d0728']['2']['content_responsiveness'] == 4

    @pytest.mark.skipif(not os.path.exists(_task2_metrics_file_path), reason='DUC 2007 task 1 metrics file does not exist')
    def test_task2_metrics(self):
        summary_level_metrics = JsonlReader(_task2_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files. Unlike
        # the "system_level_test.py", these metrics don't have aggregated values in the NIST data,
        # so we just check to make sure a few of them were correctly parsed
        # updateEval/Pyramid/scoring/2007_modified_scores.txt
        assert summary_level_metrics['d0703-A']['39']['modified_pyramid_score'] == pytest.approx(0.2857, 1e-2)
        assert summary_level_metrics['d0711-B']['45']['modified_pyramid_score'] == pytest.approx(0.2353, 1e-2)
        assert summary_level_metrics['d0726-C']['51']['modified_pyramid_score'] == pytest.approx(0.1739, 1e-2)

        assert summary_level_metrics['d0703-A']['39']['num_scus'] == 3
        assert summary_level_metrics['d0711-B']['45']['num_scus'] == 1
        assert summary_level_metrics['d0726-C']['51']['num_scus'] == 2

        assert summary_level_metrics['d0703-A']['39']['num_repetitions'] == 0
        assert summary_level_metrics['d0711-B']['45']['num_repetitions'] == 0
        assert summary_level_metrics['d0726-C']['51']['num_repetitions'] == 0

        assert summary_level_metrics['d0703-A']['39']['content_responsiveness'] == 2
        assert summary_level_metrics['d0711-B']['45']['content_responsiveness'] == 2
        assert summary_level_metrics['d0726-C']['51']['content_responsiveness'] == 2



import os
import pytest

from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.metrics import Bleurt


@pytest.mark.skipif('BLEURT_ENV' not in os.environ, reason='BLEURT python environment environment variable not set')
class TestBleurt(ReferenceBasedMetricTestCase):
    def test_bleurt(self):
        # This is a regression test, not necessarily a test for correctness
        metric = Bleurt(environment_name=os.environ['BLEURT_ENV'])
        expected_output = [
            {'bleurt': {'average': -1.0048247178395588, 'max': -0.9933006763458252}},
            {'bleurt': {'average': -1.0668554306030273, 'max': -1.0025169849395752}},
            {'bleurt': {'average': -0.7564655840396881, 'max': -0.5489686727523804}},
            {'bleurt': {'average': -0.6693709492683411, 'max': -0.39580726623535156}},
            {'bleurt': {'average': -0.6533156832059225, 'max': -0.5730574727058411}},
            {'bleurt': {'average': -0.4237842659155528, 'max': -0.25567689538002014}},
            {'bleurt': {'average': -0.6848879158496857, 'max': -0.6829712390899658}},
            {'bleurt': {'average': -0.5012103617191315, 'max': -0.30445921421051025}},
            {'bleurt': {'average': -0.6940024892489115, 'max': -0.6559309959411621}},
            {'bleurt': {'average': -0.6693291465441386, 'max': -0.6304700374603271}},
            {'bleurt': {'average': -0.8384850323200226, 'max': -0.7783546447753906}},
            {'bleurt': {'average': -0.5458722561597824, 'max': -0.37889066338539124}}
        ]
        super().assert_expected_output(metric, expected_output)

    def test_bleurt_order_invariant(self):
        metric = Bleurt(environment_name=os.environ['BLEURT_ENV'])
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['bleurt'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'bleurt'])


import math
from collections import Counter
from typing import List, Tuple

from sacrerouge.data import MetricsDict, Pyramid, PyramidAnnotation
from sacrerouge.data.jackknifers import PyramidJackknifer
from sacrerouge.metrics import Metric


@Metric.register('pyramid-score')
class PyramidScore(Metric):
    """
    Implements calculating the modified pyramid score based on the description from page 7
    of https://www.cis.upenn.edu/~nenkova/papers/p1-nenkova.pdf and section 3 of
    http://www.cs.columbia.edu/nlp/papers/2005/passonneau_al_05.pdf.
    """
    def __init__(self, name_override: str = None):
        super().__init__(['annotation'], ['pyramid'], PyramidJackknifer())
        self.name = name_override or 'modified_pyramid_score'

    def score(self, annotation: PyramidAnnotation, pyramid: Pyramid) -> MetricsDict:
        # Create a mapping from the SCU id to its weight and count how many are at each weight
        scu_id_to_weight = {}
        weight_to_num_scus = Counter()
        for scu in pyramid.scus:
            weight = scu.get_weight()
            scu_id_to_weight[scu.scu_id] = weight
            weight_to_num_scus[weight] += 1

        # Calculate the total weight of the SCUs in the annotation
        total_weight = 0
        for scu in annotation.scus:
            # It's possible the SCU id isn't in the Pyramid, for example, if we are
            # doing jackknifing and the reference corresponding to an SCU of weight 1 was removed
            if scu.scu_id in scu_id_to_weight:
                total_weight += scu_id_to_weight[scu.scu_id]

        # Calculate the average number of SCUs in the pyramid summaries
        total_scus = 0
        for i in range(len(pyramid.summarizer_ids)):
            total_scus += len(pyramid.get_scu_id_set(i))
        average_num_scus = total_scus / len(pyramid.summarizer_ids)

        # Calculate the weight of an ideal summary with `average_num_scus` SCUs
        ideal_weight = 0
        scus_remaining = int(math.ceil(average_num_scus))
        for weight in sorted(weight_to_num_scus.keys(), reverse=True):
            if scus_remaining <= 0:
                break
            possible_scus = weight_to_num_scus[weight]
            num_scus_taken = min(scus_remaining, possible_scus)
            ideal_weight += num_scus_taken * weight
            scus_remaining -= num_scus_taken

        # The modified pyramid score is the ratio of the weight to the ideal weight
        return MetricsDict({self.name: total_weight / ideal_weight})

    def score_multi(self, annotations: List[PyramidAnnotation], pyramid: Pyramid) -> List[MetricsDict]:
        return self.score_multi_all([annotations], [pyramid])[0]

    def score_all(self, annotations: List[PyramidAnnotation], pyramids: List[Pyramid]) -> List[MetricsDict]:
        annotation_list = [[annotation] for annotation in annotations]
        metrics_lists = self.score_multi_all(annotation_list, pyramids)
        return [metrics_list[0] for metrics_list in metrics_lists]

    def score_multi_all(self, annotations_list: List[List[PyramidAnnotation]], pyramids: List[Pyramid]) -> List[List[MetricsDict]]:
        metrics_dict_lists = []
        for annotations, pyramid in zip(annotations_list, pyramids):
            metrics_dict_lists.append([])
            for annotation in annotations:
                metrics_dict_lists[-1].append(self.score(annotation, pyramid))
        return metrics_dict_lists

    def evaluate(self, annotations: List[PyramidAnnotation], pyramids: List[Pyramid]) -> Tuple[MetricsDict, List[MetricsDict]]:
        micro_metrics_list = self.score_all(annotations, pyramids)
        macro_metrics = self.aggregate(micro_metrics_list)
        return macro_metrics, micro_metrics_list



import argparse
import logging
import os
from collections import defaultdict
from overrides import overrides
from subprocess import Popen, PIPE
from typing import List, Tuple

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.data import MetricsDict
from sacrerouge.data.types import DocumentType, SummaryType
from sacrerouge.metrics import DocumentBasedMetric, Metric

logger = logging.getLogger(__name__)


@Metric.register('simetrix')
class SIMetrix(DocumentBasedMetric):
    def __init__(self,
                 use_stemmer: bool = True,
                 remove_stopwords: bool = True,
                 simetrix_root: str = f'{DATA_ROOT}/metrics/simetrix'):
        super().__init__(['summary'], ['documents'])
        self.use_stemmer = use_stemmer
        self.remove_stopwords = remove_stopwords
        if not os.path.exists(simetrix_root):
            raise Exception('SIMetrix directory does not exist. Please run the setup code')
        self.jar_path = f'{simetrix_root}/simetrix.jar'
        self.data_dir = f'{simetrix_root}/data'

    def _save_summary_like(self, summary: SummaryType, file_path: str) -> None:
        dirname = os.path.dirname(file_path)
        if dirname:
            os.makedirs(dirname, exist_ok=True)
        with open(file_path, 'w') as out:
            if isinstance(summary, list):
                out.write('\n'.join(summary))
            else:
                out.write(summary)

    def _parse_macro_file(self, file_path: str) -> List[MetricsDict]:
        metrics_dict = {}
        with open(file_path, 'r') as f:
            for i, line in enumerate(f):
                columns = line.split()
                if i == 0:
                    header = columns
                else:
                    index = int(columns[0])
                    metrics = MetricsDict()
                    for j, name in enumerate(header[1:]):
                        # All the names begin with "Avg"
                        name = name[3:]
                        metrics[name] = float(columns[j + 1])
                    metrics_dict[index] = metrics

        # Sort and return a list according to the index
        metrics_list = []
        for index in sorted(metrics_dict.keys()):
            assert index == len(metrics_list)
            metrics_list.append(metrics_dict[index])
        return metrics_list

    def _parse_micro_file(self, file_path: str) -> List[List[MetricsDict]]:
        metrics_dicts = defaultdict(dict)
        with open(file_path, 'r') as f:
            for i, line in enumerate(f):
                columns = line.split()
                if i == 0:
                    header = columns
                else:
                    instance_index = int(columns[0])
                    system_index = int(columns[1])
                    metrics = MetricsDict()
                    for j, name in enumerate(header[2:]):
                        metrics[name] = float(columns[j + 2])
                    metrics_dicts[instance_index][system_index] = metrics

        # Sort and return a list according to the index
        metrics_lists = [None] * len(metrics_dicts)
        for instance_index in sorted(metrics_dicts.keys()):
            metrics_dict = metrics_dicts[instance_index]
            metrics_lists[instance_index] = [None] * len(metrics_dict)
            for system_index in sorted(metrics_dict.keys()):
                metrics_lists[instance_index][system_index] = metrics_dict[system_index]
        return metrics_lists

    def _run(self,
             summaries_list: List[List[SummaryType]],
             documents_list: List[List[str]]) -> Tuple[List[MetricsDict], List[List[MetricsDict]]]:
        with TemporaryDirectory() as temp_dir:
            mappings_file_path = f'{temp_dir}/mappings.txt'
            with open(mappings_file_path, 'w') as out:
                for i, (summaries, documents) in enumerate(zip(summaries_list, documents_list)):
                    document_dir = f'{temp_dir}/documents/{i}'
                    for j, document in enumerate(documents):
                        document_file_path = f'{document_dir}/{j}.txt'
                        self._save_summary_like(document, document_file_path)

                    for j, summary in enumerate(summaries):
                        summary_file_path = f'{temp_dir}/summaries/{i}-{j}.txt'
                        self._save_summary_like(summary, summary_file_path)

                        out.write(f'{i} {j} {document_dir} {summary_file_path}\n')

            config_file_path = f'{temp_dir}/config'
            with open(config_file_path, 'w') as out:
                perform_stemming = 'Y' if self.use_stemmer else 'N'
                out.write(f'performStemming = {perform_stemming}\n')

                remove_stopwords = 'Y' if self.remove_stopwords else 'N'
                out.write(f'removeStopWords = {remove_stopwords}\n')
                out.write(f'stopFilePath = {self.data_dir}/smart_common_words.txt\n')

                out.write(f'divergence = Y\n')
                out.write(f'frequencyFeatures = Y\n')
                out.write(f'cosineOverlap = Y\n')
                out.write(f'topicWordFeatures = Y\n')
                out.write(f'backgroundCorpusFreqCounts = {self.data_dir}/bgFreqCounts.unstemmed.txt\n')
                out.write(f'backgroundIdfUnstemmed = {self.data_dir}/bgIdfValues.unstemmed.txt\n')
                out.write(f'backgroundIdfStemmed = {self.data_dir}/bgIdfValues.stemmed.txt\n')

            command = [
                'java',
                '-cp', self.jar_path,
                'edu.upenn.seas.simetrix.InputBasedEvaluation',
                mappings_file_path,
                config_file_path
            ]

            logger.info(f'Running SIMetrix command: "{command}"')
            process = Popen(command, stdout=PIPE, stderr=PIPE)
            stdout, stderr = process.communicate()
            if stderr:
                raise Exception(f'SIMetrix failed with stderr: {stderr.decode()}')

            macro_results = self._parse_macro_file(f'{temp_dir}/mappings.txt.ieval.macro')
            micro_results = self._parse_micro_file(f'{temp_dir}/mappings.txt.ieval.micro')
            return macro_results, micro_results

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        documents_list: List[List[DocumentType]]) -> List[List[MetricsDict]]:
        _, micro_metrics_lists = self._run(summaries_list, documents_list)
        return micro_metrics_lists

    def evaluate(self,
                 summaries: List[SummaryType],
                 documents_list: List[List[DocumentType]]) -> Tuple[MetricsDict, List[MetricsDict]]:
        summaries_list = [[summary] for summary in summaries]
        macro_metrics_list, micro_metrics_lists = self._run(summaries_list, documents_list)

        macro_metrics = macro_metrics_list[0]
        micro_metrics_list = [metrics_list[0] for metrics_list in micro_metrics_lists]
        return macro_metrics, micro_metrics_list


@MetricSetupSubcommand.register('simetrix')
class SIMetrixSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the SIMetrix metric'
        self.parser = parser.add_parser('simetrix', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        commands = [
            f'mkdir -p {DATA_ROOT}/metrics',
            f'cd {DATA_ROOT}/metrics',
            f'git clone https://github.com/igorbrigadir/simetrix',
            f'cd simetrix',
            f'mvn package',
            f'cp target/simetrix-1.0-SNAPSHOT.jar simetrix.jar',
            f'cp -r src/main/resources/data .',
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()

        if process.returncode == 0:
            print('SIMetrix setup success')
        else:
            print('SIMetrix setup failure')



import os
import pytest
import unittest
from collections import defaultdict

from sacrerouge.data import Pyramid, PyramidAnnotation
from sacrerouge.io import JsonlReader


_pyramids_file_path = 'datasets/duc-tac/tac2010/v1.0/task1.A-B.pyramids.jsonl'
_pyramid_annotations_file_path = 'datasets/duc-tac/tac2010/v1.0/task1.A-B.pyramid-annotations.jsonl'


@pytest.mark.skipif(not os.path.exists(_pyramids_file_path), reason='TAC 2010 Pyramids file does not exist')
@pytest.mark.skipif(not os.path.exists(_pyramid_annotations_file_path), reason='TAC 2010 Pyramid annotations file does not exist')
class TestTAC2010Pyramids(unittest.TestCase):
    def test_pyramids(self):
        """Do some basic sanity tests on the files."""
        pyramids = JsonlReader(_pyramids_file_path, Pyramid).read()
        annotations = JsonlReader(_pyramid_annotations_file_path, PyramidAnnotation).read()

        instance_id_to_pyramid = {}
        for pyramid in pyramids:
            instance_id_to_pyramid[pyramid.instance_id] = pyramid

        instance_id_to_annotations = defaultdict(list)
        for annotation in annotations:
            instance_id_to_annotations[annotation.instance_id].append(annotation)

        assert instance_id_to_pyramid.keys() == instance_id_to_annotations.keys()
        for instance_id, pyramid in instance_id_to_pyramid.items():
            assert len(pyramid.summaries) == 4
            assert len(pyramid.summarizer_ids) == 4
            for reference in pyramid.summaries:
                assert len(reference) > 0

            scu_ids = set([scu.scu_id for scu in pyramid.scus])
            for annotation in instance_id_to_annotations[instance_id]:
                assert len(annotation.summary) > 0, (instance_id, annotation.summarizer_id)
                for scu in annotation.scus:
                    assert scu.scu_id in scu_ids, (scu.scu_id, scu_ids)

    def _test_example(self, pyramid: Pyramid, annotation: PyramidAnnotation) -> None:
        scu_to_label = {scu.scu_id: scu.label for scu in pyramid.scus}
        for scu in annotation.scus:
            assert scu.label == scu_to_label[scu.scu_id]

    def test_pyramid_examples(self):
        """Pick some random examples and test them."""
        pyramids = JsonlReader(_pyramids_file_path, Pyramid).read()
        annotations = JsonlReader(_pyramid_annotations_file_path, PyramidAnnotation).read()

        instance_id_to_pyramid = {}
        for pyramid in pyramids:
            instance_id_to_pyramid[pyramid.instance_id] = pyramid

        instance_id_to_annotations = defaultdict(dict)
        for annotation in annotations:
            instance_id_to_annotations[annotation.instance_id][annotation.summarizer_id] = annotation

        pyramid = instance_id_to_pyramid['d1001-A']
        assert pyramid.summarizer_ids == ['A', 'B', 'F', 'H']
        annotation = instance_id_to_annotations['d1001-A']['2']
        label = 'Columbine'
        assert len(annotation.scus) == 3
        assert annotation.scus[0].scu_id == 3
        assert len(annotation.scus[0].contributors) == 1
        assert annotation.scus[0].contributors[0].label == label
        assert len(annotation.scus[0].contributors[0].parts) == 1
        assert annotation.scus[0].contributors[0].parts[0].text == label
        start = annotation.scus[0].contributors[0].parts[0].start
        end = annotation.scus[0].contributors[0].parts[0].end
        assert annotation.summary[start:end] == label
        self._test_example(pyramid, annotation)

        pyramid = instance_id_to_pyramid['d1002-A']
        assert pyramid.summarizer_ids == ['A', 'B', 'C', 'D']
        annotation = instance_id_to_annotations['d1002-A']['29']
        contrib_label = 'official investigations _ bythe US Justice Department'
        part_label = 'official investigations _ by the US Justice Department'
        assert len(annotation.scus) == 1
        assert annotation.scus[0].scu_id == 11
        assert len(annotation.scus[0].contributors) == 1
        assert annotation.scus[0].contributors[0].label == contrib_label
        assert len(annotation.scus[0].contributors[0].parts) == 1
        assert annotation.scus[0].contributors[0].parts[0].text == part_label
        start = annotation.scus[0].contributors[0].parts[0].start
        end = annotation.scus[0].contributors[0].parts[0].end
        assert annotation.summary[start:end] == part_label
        self._test_example(pyramid, annotation)



import importlib
import os
import pkgutil
import sys
import urllib
from contextlib import contextmanager
from google_drive_downloader import GoogleDriveDownloader
from pathlib import Path
from shutil import which
from typing import Generator, T, Union

PathType = Union[os.PathLike, str]
ContextManagerFunctionReturnType = Generator[T, None, None]


def command_exists(command: str) -> bool:
    return which(command) is not None


def download_url_to_file(url: str, file_path: str, force: bool = False) -> None:
    """
    Downloads a url to a local file if it does not already exist. The directory
    path to the download file will be created if it does not exist.

    Parameters
    ----------
    url: ``str``, required.
        The url of the file to download.
    file_path: ``str``, required.
        The path where the file should be saved.
    force: ``bool``, optional (default = ``False``)
        If false, the file is not downloaded if it exists. Otherwise, the
        file will be downloaded no matter what.
    """
    dirname = os.path.dirname(file_path)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    if os.path.exists(file_path) and not force:
        print(f'Skipping downloading {url}')
        return

    print(f'Downloading {url} to {file_path}')
    urllib.request.urlretrieve(url, file_path)


def download_file_from_google_drive(file_id: str, file_path: str, force: bool = False) -> None:
    dirname = os.path.dirname(file_path)
    if dirname:
        os.makedirs(dirname, exist_ok=True)

    if os.path.exists(file_path) and not force:
        print(f'Skipping downloading file {file_id}')
        return

    print(f'Downloading file {file_id} to {file_path}')
    GoogleDriveDownloader.download_file_from_google_drive(file_id, file_path, overwrite=True)


@contextmanager
def push_python_path(path: PathType) -> ContextManagerFunctionReturnType[None]:
    """
    Prepends the given path to `sys.path`.
    This method is intended to use with `with`, so after its usage, its value willbe removed from
    `sys.path`.
    """
    # In some environments, such as TC, it fails when sys.path contains a relative path, such as ".".
    path = Path(path).resolve()
    path = str(path)
    sys.path.insert(0, path)
    try:
        yield
    finally:
        # Better to remove by value, in case `sys.path` was manipulated in between.
        sys.path.remove(path)


# Borrowed from AllenNLP
def import_module_and_submodules(package_name: str) -> None:
    """
    Import all submodules under the given package.
    Primarily useful so that people using AllenNLP as a library
    can specify their own custom packages and have their custom
    classes get loaded and registered.
    """
    importlib.invalidate_caches()

    # For some reason, python doesn't always add this by default to your path, but you pretty much
    # always want it when using `--include-package`.  And if it's already there, adding it again at
    # the end won't hurt anything.
    with push_python_path("."):
        # Import at top level
        module = importlib.import_module(package_name)
        path = getattr(module, "__path__", [])
        path_string = "" if not path else path[0]

        # walk_packages only finds immediate children, so need to recurse.
        for module_finder, name, _ in pkgutil.walk_packages(path):
            # Sometimes when you import third-party libraries that are on your path,
            # `pkgutil.walk_packages` returns those too, so we need to skip them.
            if path_string and module_finder.path != path_string:
                continue
            subpackage = f"{package_name}.{name}"
            import_module_and_submodules(subpackage)


import os
import pytest

from sacrerouge.common.testing.metric_test_cases import ReferenceBasedMetricTestCase
from sacrerouge.common.testing.util import sacrerouge_command_exists
from sacrerouge.metrics import PyrEval


@pytest.mark.skipif('PYREVAL_ENV' not in os.environ, reason='PyrEval python environment environment variable not set')
class TestPyrEval(ReferenceBasedMetricTestCase):
    def test_pyreval(self):
        # This is a regression test, not necessarily a test for correctness
        metric = PyrEval(environment_name=os.environ['PYREVAL_ENV'])
        expected_output = [
            {'pyreval': {'raw': 12, 'quality': 0.375, 'coverage': 0.2553191489361702, 'comprehensive': 0.31515957446808507}},
            {'pyreval': {'raw': 16, 'quality': 0.47058823529411764, 'coverage': 0.3404255319148936, 'comprehensive': 0.4055068836045056}},
            {'pyreval': {'raw': 12, 'quality': 0.631578947368421, 'coverage': 0.6666666666666666, 'comprehensive': 0.6491228070175439}},
            {'pyreval': {'raw': 8, 'quality': 0.2857142857142857, 'coverage': 0.5, 'comprehensive': 0.39285714285714285}},
            {'pyreval': {'raw': 17, 'quality': 0.4857142857142857, 'coverage': 0.5, 'comprehensive': 0.4928571428571429}},
            {'pyreval': {'raw': 17, 'quality': 0.4857142857142857, 'coverage': 0.5, 'comprehensive': 0.4928571428571429}},
            {'pyreval': {'raw': 11, 'quality': 0.5789473684210527, 'coverage': 0.7857142857142857, 'comprehensive': 0.6823308270676691}},
            {'pyreval': {'raw': 11, 'quality': 0.7857142857142857, 'coverage': 0.7333333333333333, 'comprehensive': 0.7595238095238095}},
            {'pyreval': {'raw': 9, 'quality': 0.2903225806451613, 'coverage': 0.2571428571428571, 'comprehensive': 0.2737327188940092}},
            {'pyreval': {'raw': 19, 'quality': 0.5277777777777778, 'coverage': 0.5428571428571428, 'comprehensive': 0.5353174603174603}},
            {'pyreval': {'raw': 10, 'quality': 0.625, 'coverage': 0.625, 'comprehensive': 0.625}},
            {'pyreval': {'raw': 8, 'quality': 0.4, 'coverage': 0.5333333333333333, 'comprehensive': 0.4666666666666667}}
        ]
        super().assert_expected_output(metric, expected_output)

    def test_pyreval_order_invariant(self):
        metric = PyrEval(environment_name=os.environ['PYREVAL_ENV'])
        self.assert_order_invariant(metric)

    def test_command_exists(self):
        assert sacrerouge_command_exists(['pyreval'])

    def test_setup_command_exists(self):
        assert sacrerouge_command_exists(['setup-metric', 'pyreval'])


import zipfile
from collections import defaultdict

from sacrerouge.io import JsonlWriter
from sacrerouge.datasets.multiling.util import LANGUAGE_CODES


def load_training_data(train_zip: str):
    documents = defaultdict(lambda: defaultdict(list))
    summaries = defaultdict(lambda: defaultdict(list))
    with zipfile.ZipFile(train_zip, 'r') as zip:
        for file_path in zip.namelist():
            if file_path.endswith('.250'):
                path = file_path.split('/')
                language_code = LANGUAGE_CODES[path[1]]
                filename = path[3]
                parts = filename.split('.')
                instance_id = parts[0]
                annotator = parts[1]

                contents = zip.open(file_path, 'r').read().decode()
                text = list(filter(None, [line.strip() for line in contents.splitlines()]))
                summaries[language_code][instance_id].append({
                    'annotator': annotator,
                    'text': text
                })

            elif '/M00' in file_path:
                path = file_path.split('/')
                language_code = LANGUAGE_CODES[path[1]]
                filename = path[3]
                parts = filename.split('.')
                instance_id = parts[0][:-1]

                contents = zip.open(file_path, 'r').read().decode()
                text = list(filter(None, [line.strip() for line in contents.splitlines()]))
                documents[language_code][instance_id].append({
                    'filename': filename,
                    'text': text
                })

    return documents, summaries


def load_testing_data(test_zip: str):
    documents = defaultdict(lambda: defaultdict(list))
    with zipfile.ZipFile(test_zip, 'r') as zip:
        for file_path in zip.namelist():
            if '/M00' in file_path:
                path = file_path.split('/')
                language_code = LANGUAGE_CODES[path[1]]
                filename = path[2]
                parts = filename.split('.')
                instance_id = parts[0][:-1]

                if instance_id in ['M001', 'M002', 'M003']:
                    # According to the Wiki, these are not taken into account during evaluation
                    # because these are the training topics
                    # http://multiling.iit.demokritos.gr/pages/view/1540/task-mms-multi-document-summarization-data-and-information
                    continue

                contents = zip.open(file_path, 'r').read().decode()
                text = list(filter(None, [line.strip() for line in contents.splitlines()]))
                documents[language_code][instance_id].append({
                    'filename': filename,
                    'text': text
                })
    return documents


def save_data(train_documents, train_summaries, test_documents, output_dir):
    for language in train_documents.keys():
        with JsonlWriter(f'{output_dir}/{language}.train.jsonl') as out:
            for instance_id in sorted(train_documents[language].keys()):
                out.write({
                    'instance_id': instance_id,
                    'documents': train_documents[language][instance_id],
                    'summaries': train_summaries[language][instance_id]
                })

        with JsonlWriter(f'{output_dir}/{language}.test.jsonl') as out:
            for instance_id in sorted(test_documents[language].keys()):
                out.write({
                    'instance_id': instance_id,
                    'documents': test_documents[language][instance_id]
                })


def setup(train_zip: str, test_zip: str, output_dir: str):
    train_documents, train_summaries = load_training_data(train_zip)
    test_documents = load_testing_data(test_zip)
    save_data(train_documents, train_summaries, test_documents, output_dir)


import argparse
import tarfile
from collections import defaultdict
from typing import Dict

from sacrerouge.data import Pyramid, PyramidAnnotation
from sacrerouge.io import JsonlWriter


def load_main_pyramids(main_pyramid_tar: str) -> Dict[str, Pyramid]:
    pyramids = {}
    with tarfile.open(main_pyramid_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('mainPyramidEval/allpyramids'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('.')
                instance_id = parts[0].split('-')[0].lower()

                xml = tar.extractfile(member).read().decode()
                pyramid = Pyramid.from_xml(f'{instance_id}', xml, default_document_regex='[-]*\n D[0-9]*\.M\.250\.[A-Z]\.[A-Z]\n[-]*\n')
                pyramids[instance_id] = pyramid

    return pyramids


def load_main_annotations(main_pyramid_tar: str, pyramids: Dict[str, Pyramid]) -> Dict[str, Dict[str, PyramidAnnotation]]:
    annotations = defaultdict(dict)
    with tarfile.open(main_pyramid_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('mainPyramidEval/allpans'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('.')
                instance_id = parts[0].lower()
                summarizer_id = parts[4]
                # This directory only has peers
                assert not summarizer_id.isalpha()
                summarizer_type = 'peer'

                pyramid = pyramids[instance_id]
                xml = tar.extractfile(member).read().decode()
                annotation = PyramidAnnotation.from_xml(f'{instance_id}', summarizer_id, summarizer_type, xml, pyramid)
                if annotation:
                    annotations[instance_id][summarizer_id] = annotation
                else:
                    print(f'Annotation for {instance_id}, {summarizer_id} is `None`. Skipping')

    return annotations


def load_update_pyramids(update_tar: str) -> Dict[str, Dict[str, Pyramid]]:
    pyramids = defaultdict(dict)
    with tarfile.open(update_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('updateEval/Pyramid/allpyramids'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('-')
                instance_id = parts[0].lower()
                group = parts[1]

                xml = tar.extractfile(member).read().decode()
                pyramid = Pyramid.from_xml(f'{instance_id}-{group}', xml, default_document_regex='[-]*\n D[0-9]*\.M\.250\.[A-Z]\.[A-Z]\n[-]*\n')
                pyramids[instance_id][group] = pyramid

    return pyramids


def load_update_annotations(update_tar: str, pyramids: Dict[str, Dict[str, Pyramid]]) -> Dict[str, Dict[str, Dict[str, PyramidAnnotation]]]:
    annotations = defaultdict(lambda: defaultdict(dict))
    with tarfile.open(update_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('updateEval/Pyramid/allpans'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('.')
                instance_id = parts[0].split('-')[0].lower()
                group = parts[0].split('-')[1]
                summarizer_id = parts[4]
                # This directory only has peers
                assert not summarizer_id.isalpha()
                summarizer_type = 'peer'

                pyramid = pyramids[instance_id][group]
                xml = tar.extractfile(member).read().decode()
                annotation = PyramidAnnotation.from_xml(f'{instance_id}-{group}', summarizer_id, summarizer_type, xml, pyramid)
                if annotation:
                    annotations[instance_id][summarizer_id][group] = annotation
                else:
                    print(f'Annotation for {instance_id}-{group}, {summarizer_id} is `None`. Skipping')

    return annotations


def save_main_pyramids(pyramids: Dict[str, Pyramid],
                       output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task1.pyramids.jsonl') as out:
        for instance_id in sorted(pyramids.keys()):
            pyramid = pyramids[instance_id]
            out.write(pyramid)


def save_main_pyramid_annotations(annotations: Dict[str, Dict[str, PyramidAnnotation]],
                                  output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task1.pyramid-annotations.jsonl') as out:
        for instance_id in sorted(annotations.keys()):
            for summarizer_id in sorted(annotations[instance_id].keys()):
                annotation = annotations[instance_id][summarizer_id]
                out.write(annotation)


def save_update_pyramids(pyramids: Dict[str, Dict[str, Pyramid]],
                         output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task2.A.pyramids.jsonl') as out_A:
        with JsonlWriter(f'{output_dir}/task2.B.pyramids.jsonl') as out_B:
            with JsonlWriter(f'{output_dir}/task2.C.pyramids.jsonl') as out_C:
                with JsonlWriter(f'{output_dir}/task2.A-B-C.pyramids.jsonl') as out_A_B_C:
                    for instance_id in sorted(pyramids.keys()):
                        pyramid_A = pyramids[instance_id]['A']
                        pyramid_B = pyramids[instance_id]['B']
                        pyramid_C = pyramids[instance_id]['C']
                        out_A_B_C.write(pyramid_A)
                        out_A_B_C.write(pyramid_B)
                        out_A_B_C.write(pyramid_C)
                        out_A.write(pyramid_A)
                        out_B.write(pyramid_B)
                        out_C.write(pyramid_C)


def save_update_annotations(annotations: Dict[str, Dict[str, Dict[str, PyramidAnnotation]]],
                            output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task2.A.pyramid-annotations.jsonl') as out_A:
        with JsonlWriter(f'{output_dir}/task2.B.pyramid-annotations.jsonl') as out_B:
            with JsonlWriter(f'{output_dir}/task2.C.pyramid-annotations.jsonl') as out_C:
                with JsonlWriter(f'{output_dir}/task2.A-B-C.pyramid-annotations.jsonl') as out_A_B_C:
                    for instance_id in sorted(annotations.keys()):
                        for summarizer_id in sorted(annotations[instance_id].keys()):
                            pyramid_A = annotations[instance_id][summarizer_id]['A']
                            out_A_B_C.write(pyramid_A)
                            out_A.write(pyramid_A)

                            pyramid_B = annotations[instance_id][summarizer_id]['B']
                            out_A_B_C.write(pyramid_B)
                            out_B.write(pyramid_B)

                            pyramid_C = annotations[instance_id][summarizer_id]['C']
                            out_A_B_C.write(pyramid_C)
                            out_C.write(pyramid_C)


def setup(data_dir: str, output_dir: str):
    main_pyramid_tar = f'{data_dir}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/results/mainPyramidEval.tar.gz'
    update_tar = f'{data_dir}/scrapes/duc.nist.gov/past_duc_aquaint/duc2007/results/updateEval.tar.gz'
    main(main_pyramid_tar, update_tar, output_dir)


def main(main_pyramid_tar, update_tar, output_dir):
    main_pyramids = load_main_pyramids(main_pyramid_tar)
    main_pyramid_annotations = load_main_annotations(main_pyramid_tar, main_pyramids)
    save_main_pyramids(main_pyramids, output_dir)
    save_main_pyramid_annotations(main_pyramid_annotations, output_dir)

    update_pyramids = load_update_pyramids(update_tar)
    update_annotations = load_update_annotations(update_tar, update_pyramids)
    save_update_pyramids(update_pyramids, output_dir)
    save_update_annotations(update_annotations, output_dir)


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('main_pyramid_tar')
    argp.add_argument('update_tar')
    argp.add_argument('output_dir')
    args = argp.parse_args()

    main(args.main_pyramid_tar, args.update_tar, args.output_dir)



import os
import pytest
import unittest
from collections import defaultdict
from typing import Dict, List

from sacrerouge.data import Metrics
from sacrerouge.io import JsonlReader

_task1_metrics_file_path = 'datasets/duc-tac/duc2002/v1.0/task1.metrics.jsonl'
_task2_10_metrics_file_path = 'datasets/duc-tac/duc2002/v1.0/task2.10.metrics.jsonl'
_task2_50_metrics_file_path = 'datasets/duc-tac/duc2002/v1.0/task2.50.metrics.jsonl'
_task2_100_metrics_file_path = 'datasets/duc-tac/duc2002/v1.0/task2.100.metrics.jsonl'
_task2_200_metrics_file_path = 'datasets/duc-tac/duc2002/v1.0/task2.200.metrics.jsonl'


class TestDUC2002Metrics(unittest.TestCase):
    def _convert_to_dicts(self, metrics_list: List[Metrics]) -> Dict[str, Dict[str, Metrics]]:
        metrics_dicts = defaultdict(dict)
        for metrics in metrics_list:
            metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics.metrics
        return metrics_dicts

    @pytest.mark.skipif(not os.path.exists(_task1_metrics_file_path), reason='DUC 2002 task 1 metrics file does not exist')
    def test_task1_system_level(self):
        summary_level_metrics = JsonlReader(_task1_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/abstracts.phase1/short.results.table
        assert summary_level_metrics['WSJ880912-0064']['16']['length_adjusted_coverage']['median'] == 0.007
        assert summary_level_metrics['AP890925-0124']['17']['length_adjusted_coverage']['mean'] == 0.333
        assert summary_level_metrics['FT923-5589']['19']['coverage']['mean'] == 0.417

    @pytest.mark.skipif(not os.path.exists(_task2_10_metrics_file_path), reason='DUC 2002 task 2 length 10 metrics file does not exist')
    def test_task2_length10__system_level(self):
        summary_level_metrics = JsonlReader(_task2_10_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/abstracts.phase1/short.results.table
        assert summary_level_metrics['d062']['29']['length_adjusted_coverage']['median'] == 0.0
        assert summary_level_metrics['d070']['25']['num_peer_units'] == 1
        assert summary_level_metrics['d109']['16']['coverage']['mean'] == 0.2

    @pytest.mark.skipif(not os.path.exists(_task2_50_metrics_file_path), reason='DUC 2002 task 2 length 50 metrics file does not exist')
    def test_task2_length50_system_level(self):
        summary_level_metrics = JsonlReader(_task2_50_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/abstracts.phase1/short.results.table
        assert summary_level_metrics['d109']['16']['length_adjusted_coverage']['median'] == 0.047
        assert summary_level_metrics['d112']['20']['peer_quality']['Q1'] == 1
        assert summary_level_metrics['d114']['19']['coverage']['mean'] == 0.25

    @pytest.mark.skipif(not os.path.exists(_task2_100_metrics_file_path), reason='DUC 2002 task 2 length 100 metrics file does not exist')
    def test_task2_length100_system_level(self):
        summary_level_metrics = JsonlReader(_task2_100_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/abstracts.phase1/short.results.table
        assert summary_level_metrics['d061']['16']['length_adjusted_coverage']['median'] == 0.003
        assert summary_level_metrics['d063']['25']['unmarked_units_related_to_subject'] == 0.8
        assert summary_level_metrics['d066']['C']['coverage']['mean'] == 0.523

    @pytest.mark.skipif(not os.path.exists(_task2_200_metrics_file_path), reason='DUC 2002 task 2 length 200 metrics file does not exist')
    def test_task2_length200_system_level(self):
        summary_level_metrics = JsonlReader(_task2_200_metrics_file_path, Metrics).read()
        summary_level_metrics = self._convert_to_dicts(summary_level_metrics)

        # Check a few metrics to make sure they are equal to what's in the NIST files
        # results/abstracts.phase1/short.results.table
        assert summary_level_metrics['d066']['20']['peer_quality']['Q5'] == 1
        assert summary_level_metrics['d069']['3']['num_peer_units'] == 7
        assert summary_level_metrics['d111']['28']['coverage']['mean'] == 0.411


import argparse
import tarfile
from collections import defaultdict
from typing import Dict

from sacrerouge.data import Pyramid, PyramidAnnotation
from sacrerouge.io import JsonlWriter


def load_pyramids(eval_tar: str) -> Dict[str, Dict[str, Pyramid]]:
    pyramids = defaultdict(dict)
    with tarfile.open(eval_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('GuidedSumm2010_eval/manual/pyramids'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('.')
                instance_id = parts[0].split('-')[0].lower()
                group = parts[0].split('-')[1]

                xml = tar.extractfile(member).read().decode()
                pyramid = Pyramid.from_xml(f'{instance_id}-{group}', xml)
                pyramids[instance_id][group] = pyramid

    return pyramids


def load_peer_pyramids(eval_tar: str, pyramids: Dict[str, Dict[str, Pyramid]]) -> Dict[str, Dict[str, Dict[str, PyramidAnnotation]]]:
    annotations = defaultdict(lambda: defaultdict(dict))
    with tarfile.open(eval_tar, 'r') as tar:
        for member in tar.getmembers():
            if member.isfile() and member.name.startswith('GuidedSumm2010_eval/manual/peers/'):
                path = member.name.split('/')
                filename = path[-1]
                parts = filename.split('.')
                instance_id = parts[0].split('-')[0].lower()
                group = parts[0].split('-')[1]
                summarizer_id = parts[-1]
                # This directory only has peers
                assert not summarizer_id.isalpha()
                summarizer_type = 'peer'

                pyramid = pyramids[instance_id][group]
                xml = tar.extractfile(member).read().decode()
                annotation = PyramidAnnotation.from_xml(f'{instance_id}-{group}', summarizer_id, summarizer_type, xml, pyramid)
                if annotation:
                    annotations[instance_id][summarizer_id][group] = annotation
                else:
                    print(f'Annotation for {instance_id}-{group}, {summarizer_id} is `None`. Skipping')

    return annotations


def save_pyramids(pyramids: Dict[str, Dict[str, Pyramid]],
                  output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task1.A.pyramids.jsonl') as out_A:
        with JsonlWriter(f'{output_dir}/task1.B.pyramids.jsonl') as out_B:
            with JsonlWriter(f'{output_dir}/task1.A-B.pyramids.jsonl') as out_A_B:
                for instance_id in sorted(pyramids.keys()):
                    pyramid_A = pyramids[instance_id]['A']
                    pyramid_B = pyramids[instance_id]['B']
                    out_A_B.write(pyramid_A)
                    out_A_B.write(pyramid_B)
                    out_A.write(pyramid_A)
                    out_B.write(pyramid_B)


def save_peer_pyramids(pyramids: Dict[str, Dict[str, Dict[str, PyramidAnnotation]]],
                       output_dir: str) -> None:
    with JsonlWriter(f'{output_dir}/task1.A.pyramid-annotations.jsonl') as out_A:
        with JsonlWriter(f'{output_dir}/task1.B.pyramid-annotations.jsonl') as out_B:
            with JsonlWriter(f'{output_dir}/task1.A-B.pyramid-annotations.jsonl') as out_A_B:
                for instance_id in sorted(pyramids.keys()):
                    for summarizer_id in sorted(pyramids[instance_id].keys()):
                        if 'A' in pyramids[instance_id][summarizer_id]:
                            pyramid_A = pyramids[instance_id][summarizer_id]['A']
                            out_A_B.write(pyramid_A)
                            out_A.write(pyramid_A)

                        if 'B' in pyramids[instance_id][summarizer_id]:
                            pyramid_B = pyramids[instance_id][summarizer_id]['B']
                            out_A_B.write(pyramid_B)
                            out_B.write(pyramid_B)


def setup(data_dir: str, output_dir: str):
    eval_tar = f'{data_dir}/scrapes/tac.nist.gov/protected/past-aquaint-aquaint2/2010/GuidedSumm2010_eval.tgz'
    main(eval_tar, output_dir)


def main(eval_tar, output_dir):
    pyramids = load_pyramids(eval_tar)
    save_pyramids(pyramids, output_dir)

    peer_pyramids = load_peer_pyramids(eval_tar, pyramids)
    save_peer_pyramids(peer_pyramids, output_dir)


if __name__ == '__main__':
    argp = argparse.ArgumentParser()
    argp.add_argument('eval_tar')
    argp.add_argument('output_dir')
    args = argp.parse_args()

    main(args.eval_tar, args.output_dir)



import os
import pytest
import unittest

from sacrerouge.data import Pyramid
from sacrerouge.io import JsonlReader

_pyramids_file_path = 'datasets/duc-tac/tac2008/v1.0/task1.A.pyramids.jsonl'


@pytest.mark.skipif(not os.path.exists(_pyramids_file_path), reason='TAC 2008 pyramids file does not exist')
class TestPyramid(unittest.TestCase):
    def setUp(self):
        self.pyramid = None
        for pyramid in JsonlReader(_pyramids_file_path, Pyramid).read():
            if pyramid.instance_id == 'd0801-A':
                self.pyramid = pyramid
                break

    def test_get_scu_id_set(self):
        assert self.pyramid.get_scu_id_set(0) == set([11, 14, 15, 16, 22, 23, 25, 27, 29, 32, 33, 47])
        assert self.pyramid.get_scu_id_set(1) == set([11, 12, 21, 32, 35, 36, 37, 38, 39, 43, 46])
        assert self.pyramid.get_scu_id_set(2) == set([11, 12, 14, 15, 16, 19, 20, 24, 29, 32, 35])
        assert self.pyramid.get_scu_id_set(3) == set([11, 12, 14, 19, 21, 22, 23, 27, 39, 42, 44, 45])

    def test_remove_summary(self):
        pyramid = self.pyramid.remove_summary(0)
        assert pyramid.summarizer_ids == ['C', 'E', 'G']
        assert len(pyramid.summaries) == 3
        assert pyramid.summaries[0].startswith('Emirate Airlines ordered the first passenger')
        assert pyramid.summaries[1].startswith('The largest passenger airliner ever built')
        assert pyramid.summaries[2].startswith('In 1994 Airbus began engineering')

        assert pyramid.get_scu_id_set(0) == set([11, 12, 21, 32, 35, 36, 37, 38, 39, 43, 46])
        assert pyramid.get_scu_id_set(1) == set([11, 12, 14, 15, 16, 19, 20, 24, 29, 32, 35])
        assert pyramid.get_scu_id_set(2) == set([11, 12, 14, 19, 21, 22, 23, 27, 39, 42, 44, 45])

        # Make sure some old SCUs unique to the removed summary aren't there
        scu_ids = set([scu.scu_id for scu in pyramid.scus])
        assert 33 not in scu_ids
        assert 47 not in scu_ids

    def test_get_annotation(self):
        annotation = self.pyramid.get_annotation(0)
        assert annotation.instance_id == self.pyramid.instance_id
        assert annotation.summarizer_id == self.pyramid.summarizer_ids[0]
        assert annotation.summarizer_type == 'reference'
        assert annotation.summary.startswith('The European Airbus A380 flew its maiden')
        assert len(annotation.scus) == 12
        assert annotation.get_scu_id_set() == set([11, 14, 15, 16, 22, 23, 25, 27, 29, 32, 33, 47])



import argparse
import itertools
import json
import logging
import matplotlib.pyplot as plt
import numpy as np
import os
import warnings
from collections import defaultdict
from overrides import overrides
from scipy.stats import kendalltau, pearsonr, spearmanr
from typing import Any, Dict, List, Tuple, Union

from sacrerouge.commands import RootSubcommand
from sacrerouge.common.logging import prepare_global_logging
from sacrerouge.data import Metrics, MetricsDict
from sacrerouge.io import JsonlReader

logger = logging.getLogger(__name__)


def _load_metrics(metrics_files: List[str]) -> List[Metrics]:
    logger.info(f'Loading metrics from {metrics_files}')
    metrics_list = []
    for metrics_file in metrics_files:
        metrics_list.extend(JsonlReader(metrics_file, Metrics).read())
    logger.info(f'Loaded {len(metrics_list)} metrics objects')
    return metrics_list


def _merge_metrics(metrics_list: List[Metrics]) -> List[Metrics]:
    logger.info(f'Merging multiple metrics objects for the same (instance, summary) into a single object')
    metrics_dicts = defaultdict(dict)
    for metrics in metrics_list:
        if metrics.summarizer_id not in metrics_dicts[metrics.instance_id]:
            metrics_dicts[metrics.instance_id][metrics.summarizer_id] = metrics
        else:
            metrics_dicts[metrics.instance_id][metrics.summarizer_id].merge(metrics)

    merged_metrics_list = []
    for metrics_dict in metrics_dicts.values():
        merged_metrics_list.extend(list(metrics_dict.values()))
    logger.info(f'Merged into {len(merged_metrics_list)} instances')
    return merged_metrics_list


def _filter_metrics(metrics_list: List[Metrics], summarizer_type: str, metric1: str, metric2: str) -> List[Metrics]:
    logger.info(f'Filtering instances to summarizer type "{summarizer_type}" and metrics "{metric1}" and "{metric2}"')

    filtered = []
    incorrect_type = 0
    missing_metric1 = 0
    missing_metric2 = 0
    for metrics in metrics_list:
        if summarizer_type == 'all' or summarizer_type == metrics.summarizer_type:
            if metric1 not in metrics.metrics:
                missing_metric1 += 1
            elif metric2 not in metrics.metrics:
                missing_metric2 += 1
            else:
                filtered.append(metrics)
        else:
            incorrect_type += 1

    if incorrect_type > 0:
        logger.info(f'Filtered {incorrect_type} instances that were not summarizer type "{summarizer_type}"')
    if missing_metric1 > 0:
        logger.info(f'Filtered {missing_metric1} instances that did not have metric "{metric1}"')
    if missing_metric2 > 0:
        logger.info(f'Filtered {missing_metric2} instances that did not have metric "{metric2}"')
    logger.info(f'{len(filtered)} instances remain after filtering')

    return filtered


def aggregate_metrics(metrics_list: List[Metrics]) -> Dict[str, MetricsDict]:
    # The instances must be sorted by the key in order to use itertools.groupby
    metrics_list = sorted(metrics_list, key=lambda metrics: metrics.summarizer_id)
    key_to_metrics = {}
    for key, group in itertools.groupby(metrics_list, lambda metrics: metrics.summarizer_id):
        group_metrics = [member.metrics for member in group]
        key_to_metrics[key] = sum(group_metrics) / len(group_metrics)
    return key_to_metrics


def compute_summary_level_correlations(metrics_list: List[Metrics],
                                       metric1: str,
                                       metric2: str) -> Tuple[Dict[str, float], Dict[str, Dict[str, float]]]:
    pearsons = {}
    spearmans = {}
    kendalls = {}
    num_nan = 0
    metrics_list = sorted(metrics_list, key=lambda metrics: metrics.instance_id)
    for instance_id, group in itertools.groupby(metrics_list, key=lambda metrics: metrics.instance_id):
        group = list(group)
        values1 = [member.metrics[metric1] for member in group]
        values2 = [member.metrics[metric2] for member in group]

        with warnings.catch_warnings():
            warnings.filterwarnings('ignore')
            r, _ = pearsonr(values1, values2)
            rho, _ = spearmanr(values1, values2)
            tau, _ = kendalltau(values1, values2)

            if any(np.isnan([r, rho, tau])):
                num_nan += 1
            else:
                pearsons[instance_id] = r
                spearmans[instance_id] = rho
                kendalls[instance_id] = tau

    if num_nan > 0:
        logger.warning(f'Skipped {num_nan} summary-level correlations because they were NaN')

    num_valid = len(pearsons)
    if num_valid > 0:
        pearson = sum(pearsons.values()) / len(pearsons)
        spearman = sum(spearmans.values()) / len(spearmans)
        kendall = sum(kendalls.values()) / len(kendalls)
    else:
        pearson, spearman, kendall = 0, 0, 0

    averaged_correlations = {
        'pearson': {'r': pearson},
        'spearman': {'rho': spearman},
        'kendall': {'tau': kendall},
        'num_summary_groups': num_valid
    }
    individual_correlations = {
        'pearson': pearsons,
        'spearman': spearmans,
        'kendall': kendalls
    }

    return averaged_correlations, individual_correlations


def compute_system_level_correlations(metrics_list: List[Metrics],
                                      metric1: str,
                                      metric2: str) -> Dict[str, float]:
    metrics_list = list(aggregate_metrics(metrics_list).values())

    values1 = [metrics[metric1] for metrics in metrics_list]
    values2 = [metrics[metric2] for metrics in metrics_list]

    r, r_pvalue = pearsonr(values1, values2)
    rho, rho_pvalue = spearmanr(values1, values2)
    tau, tau_pvalue = kendalltau(values1, values2)
    num_summarizers = len(metrics_list)

    return {
        'pearson': {
            'r': r,
            'p_value': r_pvalue
        },
        'spearman': {
            'rho': rho,
            'p_value': rho_pvalue
        },
        'kendall': {
            'tau': tau,
            'p_value': tau_pvalue
        },
        'num_summarizers': num_summarizers
    }


def compute_global_correlations(metrics_list: List[Metrics],
                                metric1: str,
                                metric2: str) -> Dict[str, float]:
    values1 = [metrics.metrics[metric1] for metrics in metrics_list]
    values2 = [metrics.metrics[metric2] for metrics in metrics_list]

    r, r_pvalue = pearsonr(values1, values2)
    rho, rho_pvalue = spearmanr(values1, values2)
    tau, tau_pvalue = kendalltau(values1, values2)
    num_summaries = len(metrics_list)

    return {
        'pearson': {
            'r': r,
            'p_value': r_pvalue
        },
        'spearman': {
            'rho': rho,
            'p_value': rho_pvalue
        },
        'kendall': {
            'tau': tau,
            'p_value': tau_pvalue
        },
        'num_summaries': num_summaries
    }


def _plot_values(values1: List[float],
                 values2: List[float],
                 metric1: str,
                 metric2: str,
                 label: str,
                 output_file: str) -> None:
    fig = plt.figure()
    plt.xlabel(metric1)
    plt.ylabel(metric2)
    plt.scatter(values1, values2, label=label)
    plt.legend()
    plt.tight_layout()

    dirname = os.path.dirname(output_file)
    if dirname:
        os.makedirs(dirname, exist_ok=True)
    fig.savefig(output_file)
    plt.close()


def _plot_system_level_metrics(metrics_list: List[Metrics],
                               metric1: str,
                               metric2: str,
                               output_file: str) -> None:
    metrics_list = list(aggregate_metrics(metrics_list).values())
    values1 = [metrics[metric1] for metrics in metrics_list]
    values2 = [metrics[metric2] for metrics in metrics_list]
    _plot_values(values1, values2, metric1, metric2, 'Systems', output_file)


def _plot_global_metrics(metrics_list: List[Metrics],
                         metric1: str,
                         metric2: str,
                         output_file: str) -> None:
    values1 = [metrics.metrics[metric1] for metrics in metrics_list]
    values2 = [metrics.metrics[metric2] for metrics in metrics_list]
    _plot_values(values1, values2, metric1, metric2, 'Summaries', output_file)


def compute_correlation(metrics_jsonl_files_or_metrics_list: Union[str, List[str], List[Metrics]],
                        metric1: str,
                        metric2: str,
                        summarizer_type: str,
                        return_all_summary_level: bool = False,
                        skip_summary_level: bool = False,
                        skip_system_level: bool = False,
                        skip_global: bool = False,
                        system_level_output_plot: str = None,
                        global_output_plot: str = None):
    if return_all_summary_level:
        assert not skip_summary_level, 'If `return_all_summary_level` is `True`, `skip_summary_level` must not be `True`'
    if system_level_output_plot is not None:
        assert not skip_system_level, 'If `system_level_output_plot` is not `None`, system-level correlations must be calculated'
    if global_output_plot is not None:
        assert not skip_global, 'If `global_output_plot` is not `None`, global correlations must be calculated'

    if isinstance(metrics_jsonl_files_or_metrics_list, str):
        # A single file
        metrics_list = _load_metrics([metrics_jsonl_files_or_metrics_list])
    elif isinstance(metrics_jsonl_files_or_metrics_list, list) and all(isinstance(item, str) for item in metrics_jsonl_files_or_metrics_list):
        # A list of files
        metrics_list = _load_metrics(metrics_jsonl_files_or_metrics_list)
    else:
        # A list of metrics
        assert isinstance(metrics_jsonl_files_or_metrics_list, list) and all(isinstance(item, Metrics) for item in metrics_jsonl_files_or_metrics_list)
        metrics_list = metrics_jsonl_files_or_metrics_list

    # Merge duplicate metrics objects into one
    metrics_list = _merge_metrics(metrics_list)

    for metrics in metrics_list:
        metrics.flatten_keys()

    metrics_list = _filter_metrics(metrics_list, summarizer_type, metric1, metric2)
    for metrics in metrics_list:
        metrics.select_metrics([metric1, metric2])
        metrics.average_values()

    results = {}
    if not skip_summary_level:
        summary_level, individual_summary_level = compute_summary_level_correlations(metrics_list, metric1, metric2)
        results['summary_level'] = summary_level

    if not skip_system_level:
        results['system_level'] = compute_system_level_correlations(metrics_list, metric1, metric2)
        if system_level_output_plot is not None:
            _plot_system_level_metrics(metrics_list, metric1, metric2, system_level_output_plot)

    if not skip_global:
        results['global'] = compute_global_correlations(metrics_list, metric1, metric2)
        if global_output_plot is not None:
            _plot_global_metrics(metrics_list, metric1, metric2, global_output_plot)

    if return_all_summary_level:
        results = (results, individual_summary_level)
    return results


@RootSubcommand.register('correlate')
class CorrelateSubcommand(RootSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Calculate the correlation between two different metrics'
        self.parser = parser.add_parser('correlate', description=description, help=description)
        self.parser.add_argument(
            '--metrics-jsonl-files',
            nargs='+',
            help='The jsonl files with the metric values. If the values are split across multiple files, they can all '
                 'be passed as arguments.',
            required=True
        )
        self.parser.add_argument(
            '--metrics',
            nargs=2,
            type=str,
            help='The flattened names of the two metrics that should be correlated',
            required=True
        )
        self.parser.add_argument(
            '--summarizer-type',
            choices=['all', 'reference', 'peer'],
            help='The type of summarizer which should be included in the correlation calculation',
            required=True
        )
        self.parser.add_argument(
            '--output-file',
            type=str,
            help='The json output file which will contain the final correlations'
        )
        self.parser.add_argument(
            '--log-file',
            type=str,
            help='The file where the log should be written'
        )
        self.parser.add_argument(
            '--silent',
            action='store_true',
            help='Controls whether the log should be written to stdout'
        )
        self.parser.add_argument(
            '--summary-level-correlations-output',
            type=str,
            help='The file where all of the summary-level correlations should be written'
        )
        self.parser.add_argument(
            '--skip-summary-level',
            action='store_true',
            help='Indicates the summary-level correlations should not be calculated'
        )
        self.parser.add_argument(
            '--skip-system-level',
            action='store_true',
            help='Indicates the system-level correlations should not be calculated'
        )
        self.parser.add_argument(
            '--skip-global',
            action='store_true',
            help='Indicates the global correlations should not be calculated'
        )
        self.parser.add_argument(
            '--system-level-output-plot',
            type=str,
            help='If provided, saves a plot of the system-level scores to the corresponding file'
        )
        self.parser.add_argument(
            '--global-output-plot',
            type=str,
            help='If provided, saves a plot of the global scores to the corresponding file'
        )
        self.parser.set_defaults(func=self.run)

    def run(self, args):
        prepare_global_logging(file_path=args.log_file, silent=args.silent)

        metric1, metric2 = args.metrics
        return_all_summary_level = args.summary_level_correlations_output is not None
        results = compute_correlation(args.metrics_jsonl_files, metric1, metric2, args.summarizer_type,
                                      return_all_summary_level=return_all_summary_level,
                                      skip_summary_level=args.skip_summary_level,
                                      skip_system_level=args.skip_system_level,
                                      skip_global=args.skip_global,
                                      system_level_output_plot=args.system_level_output_plot,
                                      global_output_plot=args.global_output_plot)

        # Strip off the original results from the individual summary correlations
        if return_all_summary_level:
            results, all_summary_level = results

        if args.output_file:
            dirname = os.path.dirname(args.output_file)
            if dirname:
                os.makedirs(dirname, exist_ok=True)
            with open(args.output_file, 'w') as out:
                out.write(json.dumps(results, indent=2))

        if not args.silent:
            logger.info(json.dumps(results, indent=2))

        # Save the individual summary-level correlations if the output file is provided. `all_summary_level`
        # should only be defined if `return_all_summary_level` is true
        if return_all_summary_level:
            with open(args.summary_level_correlations_output, 'w') as out:
                out.write(json.dumps(all_summary_level, indent=2))



import argparse
import logging
import os
from overrides import overrides
from subprocess import Popen, PIPE
from typing import List

from sacrerouge.commands import MetricSetupSubcommand
from sacrerouge.common import DATA_ROOT, TemporaryDirectory
from sacrerouge.data import MetricsDict
from sacrerouge.data.jackknifers import ReferencesJackknifer
from sacrerouge.data.types import ReferenceType, SummaryType
from sacrerouge.metrics import Metric, ReferenceBasedMetric

logger = logging.getLogger(__name__)


@Metric.register('bleurt')
class Bleurt(ReferenceBasedMetric):
    def __init__(self,
                 environment_name: str = None,
                 checkpoint: str = 'bleurt-base-128',
                 bleurt_root: str = f'{DATA_ROOT}/metrics/bleurt',
                 batch_size: int = 100,
                 verbose: bool = False):
        super().__init__(['summary'], ['references'], jackknifer=ReferencesJackknifer())
        self.environment_name = environment_name
        self.checkpoint = checkpoint.lower()
        self.bleurt_root = bleurt_root
        self.batch_size = batch_size
        self.verbose = verbose

        if self.environment_name is not None:
            if 'CONDA_INIT' not in os.environ:
                raise Exception('If `environment_name` is not none, environment variable "CONDA_INIT" must be set to the path to "conda.sh"')

        self.checkpoint_dir = self._maybe_download_checkpoint(self.checkpoint)

    def _maybe_download_checkpoint(self, checkpoint: str) -> str:
        checkpoints_dir = f'{self.bleurt_root}/checkpoints'
        this_checkpoint_dir = f'{checkpoints_dir}/{checkpoint}'
        if not os.path.exists(this_checkpoint_dir):
            commands = [
                f'mkdir -p {checkpoints_dir}',
                f'cd {checkpoints_dir}',
                f'wget https://storage.googleapis.com/bleurt-oss/{checkpoint}.zip',
                f'unzip {checkpoint}.zip'
            ]
            command = ' && '.join(commands)

            logger.info(f'Downloading checkpoint {checkpoint}')
            logger.info(f'Running command "{command}"')
            process = Popen(command, stdout=None, stderr=None, shell=True)
            process.communicate()

        return this_checkpoint_dir

    def score_multi_all(self,
                        summaries_list: List[List[SummaryType]],
                        references_list: List[List[ReferenceType]]) -> List[List[MetricsDict]]:
        with TemporaryDirectory() as temp_dir:
            # Save the summaries to a file. Each file has one summary per line.
            # For multiple references, each reference is used to evaluate the same
            # summary independently, so the system summary is repeated.
            candidate_file = f'{temp_dir}/candidates.txt'
            reference_file = f'{temp_dir}/references.txt'
            score_file = f'{temp_dir}/scores.txt'

            with open(candidate_file, 'w') as out_candidates:
                with open(reference_file, 'w') as out_references:
                    for summaries, references in zip(summaries_list, references_list):
                        for summary in summaries:
                            for reference in references:
                                if isinstance(summary, list):
                                    out_candidates.write(' '.join(summary) + '\n')
                                else:
                                    out_candidates.write(summary + '\n')

                                if isinstance(reference, list):
                                    out_references.write(' '.join(reference) + '\n')
                                else:
                                    out_references.write(reference + '\n')

            # Run through BLEURT
            commands = [f'cd {self.bleurt_root}']
            if self.environment_name is not None:
                commands.append(f'source {os.environ["CONDA_INIT"]}')
                commands.append(f'conda activate {self.environment_name}')
            commands.append(
                f'python -m bleurt.score '
                f'-candidate_file={candidate_file} '
                f'-reference_file={reference_file} '
                f'-bleurt_checkpoint={self.checkpoint_dir} '
                f'-scores_file={score_file} '
                f'-bleurt_batch_size={self.batch_size}'
            )
            command = ' && '.join(commands)

            logger.info(f'Running command: "{command}"')
            redirect = None if self.verbose else PIPE
            process = Popen(command, stdout=redirect, stderr=redirect, shell=True)
            process.communicate()

            # Load the results
            scores = list(map(float, open(score_file, 'r').read().splitlines()))
            metrics_lists = []
            index = 0
            for summaries, references in zip(summaries_list, references_list):
                metrics_lists.append([])
                for _ in summaries:
                    reference_scores = []
                    for _ in references:
                        reference_scores.append(scores[index])
                        index += 1
                    average = sum(reference_scores) / len(reference_scores)
                    max_ = max(reference_scores)
                    metrics_lists[-1].append(MetricsDict({
                        'bleurt': {
                            'average': average,
                            'max': max_
                        }
                    }))
            return metrics_lists


@MetricSetupSubcommand.register('bleurt')
class BleurtSetupSubcommand(MetricSetupSubcommand):
    @overrides
    def add_subparser(self, parser: argparse._SubParsersAction):
        description = 'Setup the BLEURT metric'
        self.parser = parser.add_parser('bleurt', description=description, help=description)
        self.parser.set_defaults(subfunc=self.run)

    @overrides
    def run(self, args):
        commands = [
            f'mkdir -p {DATA_ROOT}/metrics',
            f'cd {DATA_ROOT}/metrics',
            f'git clone https://github.com/google-research/bleurt'
        ]
        command = ' && '.join(commands)

        process = Popen(command, shell=True)
        process.communicate()
        if process.returncode == 0:
            # We do not install BLEURT on our own in case the user wants to use a different
            # python environment for the installation
            print(f'In order to finish setting up BLEURT, follow the instructions on the '
                  f'official repository (https://github.com/google-research/bleurt) to install '
                  f'it and verify it is correct via their unit tests. You should pip install it from '
                  f'the sacrerouge directory: {DATA_ROOT}/metrics/bleurt')
        else:
            print('BLEURT setup failure')